/*======================================================================================

	Program:		Range Imaging Toolkit (RITK)
	Module:			ritkRGBRImage.h
	Version:		$Revision: 1761 $
	Date:			$Date: 2011-09-23 17:25:41 +0200 (Fri, 23 Sep 2011) $

	Copyright (c) 2010-2011
	Pattern Recognition Lab, 
    University Erlangen-Nuremberg, Germany
    All rights reserved.

	This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See RITKCopyright.txt for more information.

	http://www5.informatik.uni-erlangen.de/research/software/ritk

========================================================================================*/

#ifndef RGBRIMAGE_H__
#define RGBRIMAGE_H__

#include "ritkCommon.h"

#include "ritkRImage.h"

namespace ritk
{

/**	@ingroup	Common
  *	@class		RGBRImage
  *	@author		Jakob Wasza
  *	@brief		Class tha extends and RImage by RGB information
  *
  *	@details
  *	Due to the importance of an RGB image payload this class provides a convenience extension to the RImage base class. \n
  *	Note that all overloaded methods will also call the superclass' implementation
  */
template<class TReal, class TRGB, unsigned int NDimension=2>
class RGBRImage : public RImage<TReal,NDimension>
{
public:
	/** @name Standard typedefs */
	//@{
	typedef RGBRImage						Self;
	typedef RImage<TReal,NDimension>		Superclass;
	typedef itk::SmartPointer<Self>			Pointer;
	typedef itk::SmartPointer<const Self>	ConstPointer;
	//@}

	/**	@name Typedefs for region,spacing,etc */
	//@{
	typedef typename Superclass::IndexType		IndexType;
	typedef typename Superclass::SizeType		SizeType;
	typedef typename Superclass::RegionType		RegionType;
	typedef typename Superclass::SpacingType	SpacingType;
	typedef typename Superclass::DirectionType	DirectionType;
	typedef typename Superclass::PointType		PointType;
	//@}

	/**	@name Typedefs for generic payloads */
	//@{
	typedef typename Superclass::GenericPayloadContainerType			GenericPayloadContainerType;
	typedef typename Superclass::GenericPayloadContainerPointer			GenericPayloadContainerPointer;
	typedef typename Superclass::GenericPayloadContainerConstPointer	GenericPayloadContainerConstPointer;
	//@}

	/**	@name Typedefs for the RGB image */
	//@{
	typedef TRGB									RGBType;
	typedef itk::Image<RGBType,NDimension>			RGBImageType;
	typedef typename RGBImageType::Pointer			RGBImagePointer;
	typedef typename RGBImageType::ConstPointer		RGBImageConstPointer;
	//@}

	/// Object creation
	itkNewMacro(Self);

	/// RTTI
	itkTypeMacro(RGBRImage, RImage);

	/// Image dimension
	itkStaticConstMacro(ImageDimension, unsigned int, Superclass::ImageDimension);

	/**	@brief	Modify the RGBRImage
	 *
	 *	@details
	 *	Modified() calls will affect this object and will be propagated to all wrapped/encapsulated sub-images. All subclass
	 *	should overload this method.
	 */
	virtual void Modified() const;

	/**	@name LargestPossibleRegion, BufferedRegion, RequestedRegion. */
	//@{
	virtual void SetLargestPossibleRegion(const RegionType &region);
	virtual void SetRequestedRegion(const RegionType &region);
	virtual void SetBufferedRegion(const RegionType &region);
	//@}

	/// Set the spacing. Will be delegated to the internal images
	virtual void SetSpacing(const SpacingType &Spacing);

	/// Set the origin. Will be delegated to the internal images
	virtual void SetOrigin(const PointType &Origin);

	/// Set the direction. Will be delegated to the internal images
	virtual void SetDirection(const DirectionType direction);

	/// Copy information from the specified data set
	virtual void CopyInformation(const DataObject *data);

	/**	@brief	Allocate the internal images (inherited from itk::ImageBase).
	 *
	 *	@details
	 *	Will allocate internal images. However, due to performance reasons, the world coordinate image will not be allocated automatically. 
	 *	Instead this should be done when the world coordinate image is explicitely requested (e.g. during ComputeRangeToWorldCoords()).
	 */
	virtual void Allocate();

	/**	@brief	Graft data from another image
	 *	@param	data	The object to graft
	 *
	 *	@details
	 *	This method will copy the meta information and graft the data from another object. 
	 *	Grafting can be seen as a shallow copy as the actual data will not be copied. 
	 */
	virtual void Graft(const DataObject *data);

	/**	@brief	Perform a deep copy
	 *	@param	data	The data to copy from
	 *
	 *	@details
	 *	This method will perform a deep copy (in contrast to the Graft() method that performs a shallow copy)
	 *	from the given object. Note that the sub images are allocated automatically. However, the world coordinate
	 *	image is only allocated if, and only if, the source object has a valid world coordinate image (i.e. allocated).
	 *	This scheme is due to performance reasons.
	 */
	virtual void DeepCopy(const DataObject *data);

	/// Disconnect the image from the pipeline
	virtual void DisconnectPipeline();

	/// Synchronize the RGB payload
	virtual void SynchronizeGenericImportImageContainers();

	/**	@name RGB image access */
	//@{
	virtual RGBImagePointer GetRGBImage();
	virtual RGBImageConstPointer GetRGBImage() const;
	virtual void SetRGBImage(RGBImagePointer RGBI);
	//@}


protected:
	/// Constructor
	RGBRImage();
	/// Destructor
	virtual ~RGBRImage();

	/// Print the object on a stream
	virtual void PrintSelf(std::ostream& os, itk::Indent indent) const;

protected:
	/// Payload idx for the RGB image
	unsigned int m_RGBDataPayloadIdx;

private:
	/// purposely not implemented
	RGBRImage(Self&); 
	/// purposely not implemented
	void operator=(const Self&); 
};

/** @name Template instantiations */
//@{
typedef RGBRImage<float,itk::Vector<unsigned char,3> > RGBRImageUCF2;
//@}	

} // namespace ritk

#endif // RGBRIMAGE_H__
